'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.bindActions = exports.initializeInputs = exports.updateAndValidate = exports.setErrors = exports.setValues = exports.validateInputs = exports.resetInputs = exports.setInputs = undefined;

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _defineProperty2 = require('babel-runtime/helpers/defineProperty');

var _defineProperty3 = _interopRequireDefault(_defineProperty2);

var _typeof2 = require('babel-runtime/helpers/typeof');

var _typeof3 = _interopRequireDefault(_typeof2);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

exports._setInputs = _setInputs;

var _pick2 = require('lodash/pick');

var _pick3 = _interopRequireDefault(_pick2);

var _assign2 = require('lodash/assign');

var _assign3 = _interopRequireDefault(_assign2);

var _reduce2 = require('lodash/reduce');

var _reduce3 = _interopRequireDefault(_reduce2);

var _forEach2 = require('lodash/forEach');

var _forEach3 = _interopRequireDefault(_forEach2);

var _isEmpty2 = require('lodash/isEmpty');

var _isEmpty3 = _interopRequireDefault(_isEmpty2);

var _keys2 = require('lodash/keys');

var _keys3 = _interopRequireDefault(_keys2);

var _isEqual2 = require('lodash/isEqual');

var _isEqual3 = _interopRequireDefault(_isEqual2);

var _mapValues2 = require('lodash/mapValues');

var _mapValues3 = _interopRequireDefault(_mapValues2);

var _actionTypes = require('./actionTypes');

var _helpers = require('../util/helpers');

var _mountPoint = require('../util/mountPoint');

var _log = require('../util/log');

var _log2 = _interopRequireDefault(_log);

var _reducers = require('../reducers');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var _promiseCache = {};

function _filterUnknownInputs(inputConfig, inputs) {
    // Only update known inputs
    return (0, _reduce3.default)(inputs, function (result, value, key) {
        if (!inputConfig[key]) {
            _log2.default.error(key + ' is not a valid input.');
        } else {
            result[key] = value;
        }
        return result;
    }, {});
}

function _setInputs(inputConfig, update) {
    var meta = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

    return {
        type: _actionTypes.SET_INPUT,
        payload: update,
        error: !!(0, _helpers.inputsWithErrors)(update),
        meta: (0, _extends3.default)({
            reduxMountPoint: (0, _mountPoint.getReduxMountPoint)(inputConfig)
        }, meta)
    };
}

function _fireChanges(inputConfig, update, inputsState, state, dispatch) {
    (0, _forEach3.default)(update, function (input, key) {
        var _ref = inputConfig[key] || {},
            onChange = _ref.onChange;

        if (onChange) {
            onChange(input, inputsState, state, dispatch);
        }
    });
}

/**
 * Sets inputs' states without validation
 *
 * Also fires onChange callbacks for changed inputs
 *
 * `update` example:
 *  {
 *      email: {
 *          value: 'asdf@asdf.com'
 *      }
 *  }
 */
function _setInputs3(inputConfig, update) {
    var meta = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

    return function (dispatch, getState) {
        var filteredUpdate = _filterUnknownInputs(inputConfig, update);
        dispatch(_setInputs(inputConfig, filteredUpdate, meta));
        if (!meta.suppressChange) {
            var state = getState();
            var inputsState = (0, _helpers.getInputsFromState)(inputConfig, state);
            _fireChanges(inputConfig, filteredUpdate, inputsState, state, dispatch);
        }
        return _promise2.default.resolve(filteredUpdate);
    };
}

exports.setInputs = _setInputs3;
function _resetInputs(inputConfig, inputKeys) {
    var meta = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

    var update = (0, _reducers.getDefaultInputs)(inputConfig);
    return _setInputs3(inputConfig, inputKeys ? (0, _pick3.default)(update, inputKeys) : update, (0, _extends3.default)({
        reset: true
    }, meta));
}

/**
 * Validates the current state of inputs corresponding to the given keys
 *
 * @param inputConfig
 * @param inputKeys {Array}
 * @returns {Thunk}
 */
exports.resetInputs = _resetInputs;
function _validateInputs(inputConfig, inputKeys) {
    var meta = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

    return function (dispatch, getState) {
        var inputsState = (0, _helpers.getInputsFromState)(inputConfig, getState());
        var keys = inputKeys || (0, _keys3.default)(inputsState);
        var inputsToValidate = (0, _reduce3.default)(keys, function (result, key) {
            var input = inputsState[key];
            if (inputConfig[key] && input) {
                result[key] = typeof input.error !== 'undefined' ? input.error : input.value;
            }
            return result;
        }, {});
        return _updateAndValidate(inputConfig, inputsToValidate, (0, _extends3.default)({}, meta, {
            validate: true
        }))(dispatch, getState);
    };
}

/**
 * Shortcut for setting just the values of the given inputs.
 *
 * @param inputsConfig
 * @param newValues {Object} key-value pair of new values
 * @param meta {Object}
 * @returns {Thunk}
 */
exports.validateInputs = _validateInputs;
function _setValues(inputsConfig) {
    var newValues = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    var meta = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

    return function (dispatch) {
        var update = (0, _mapValues3.default)(newValues, function (value) {
            return { value: value };
        });
        return dispatch(_setInputs3(inputsConfig, update, meta));
    };
}

/**
 * Add errors to the given inputKeys
 *
 * @param inputsConfig
 * @param inputKeys {Array}
 * @returns {Thunk}
 */
exports.setValues = _setValues;
function _setErrors(inputsConfig) {
    var inputKeys = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];
    var meta = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

    return function (dispatch, getState) {
        var inputsState = (0, _helpers.getInputsFromState)(inputsConfig, getState());
        var update = (0, _reduce3.default)(inputKeys, function (result, key) {
            // Keep the existing value and async validating state, overwrite
            // everything else.
            var input = (0, _pick3.default)(inputsState[key], ['value', 'validating']);
            result[key] = (0, _extends3.default)({}, input, {
                error: typeof inputsState[key].value === 'undefined' ? '' : inputsState[key].value
            });
            return result;
        }, {});

        return dispatch(_setInputs3(inputsConfig, update, meta));
    };
}

exports.setErrors = _setErrors;
function _updateAndValidate(inputConfig, update) {
    var meta = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

    return function (dispatch, getState) {
        var inputs = _filterUnknownInputs(inputConfig, update);

        if ((0, _isEmpty3.default)(inputs)) {
            return _promise2.default.resolve();
        }

        var state = getState();
        var inputsState = (0, _helpers.getInputsFromState)(inputConfig, state);
        var promises = [];

        var createNewState = function createNewState(newInputState) {
            return meta.initialize ? (0, _extends3.default)({
                pristine: true
            }, newInputState) : newInputState;
        };

        var changes = (0, _reduce3.default)(inputs, function (result, value, key) {
            var validator = inputConfig[key].validator;

            var _ref2 = inputsState[key] || {},
                prev = _ref2.value,
                currentlyValidating = _ref2.validating;

            var unchanged = (0, _isEqual3.default)(prev, value);
            var cachedValidation = unchanged && currentlyValidating && _promiseCache[key];
            var validationResult = cachedValidation ? cachedValidation : validator ? validator(value, inputsState, state, dispatch) : true;
            var hasAsync = (typeof validationResult === 'undefined' ? 'undefined' : (0, _typeof3.default)(validationResult)) === 'object' && !!validationResult.then;

            var dispatchAndReturnPromiseResult = function dispatchAndReturnPromiseResult(inputState) {
                return _setInputs3(inputConfig, (0, _defineProperty3.default)({}, key, createNewState(inputState)), meta)(dispatch, getState);
            };

            if (typeof validationResult === 'boolean' || typeof validationResult === 'string' || hasAsync || !validationResult) {
                // Or Promise
                var change = validationResult === true || hasAsync ? { // True or hasAsync, set value
                    value: value,
                    validating: currentlyValidating || hasAsync && !unchanged // Will be validating if async validator exists
                } : { // False returned, input invalid
                    value: prev,
                    error: typeof value === 'undefined' ? '' : value,
                    validating: false
                };

                // Add errorText if validator returned a string
                if (typeof validationResult === 'string') {
                    change.errorText = validationResult;
                }

                var newState = createNewState(change);

                // Only fire setInputs on state that is different than current
                if (!(0, _isEqual3.default)(newState, inputsState[key])) {
                    result[key] = newState;
                }

                if (!change.validating) {
                    promises.push(_promise2.default.resolve((0, _defineProperty3.default)({}, key, newState)));
                } else {
                    var resultPromise = cachedValidation ? cachedValidation : _promiseCache[key] = validationResult.then(
                    // Passed validation
                    function () {
                        return dispatchAndReturnPromiseResult({ value: value });
                    },
                    // Failed validation
                    function (errorText) {
                        return dispatchAndReturnPromiseResult((0, _extends3.default)({
                            value: prev,
                            error: typeof value === 'undefined' ? '' : value
                        }, typeof errorText === 'string' ? { errorText: errorText } : {}));
                    });

                    // Kick off async
                    promises.push(resultPromise);
                }
            } else {
                _log2.default.error('\n                Value returned from validator must be a \n                boolean representing valid/invalid, a string representing errorText, or a promise for performing async \n                validation. Got ' + (typeof validationResult === 'undefined' ? 'undefined' : (0, _typeof3.default)(validationResult)) + ' instead.\n            ');
            }

            return result;
        }, {});

        if (!(0, _isEmpty3.default)(changes)) {
            _setInputs3(inputConfig, changes, meta)(dispatch, getState);
        }

        return new _promise2.default(function (resolve, reject) {
            _promise2.default.all(promises).then(function (results) {
                var resultObject = _assign3.default.apply(undefined, [{}].concat((0, _toConsumableArray3.default)(results)));

                var erroredInputs = (0, _helpers.inputsWithErrors)(resultObject);
                if (erroredInputs) {
                    reject(erroredInputs);
                } else {
                    resolve(resultObject);
                }
            });
        });
    };
}

exports.updateAndValidate = _updateAndValidate;
function _initializeInputs(inputconfig, update) {
    var meta = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

    return _updateAndValidate(inputconfig, update, (0, _extends3.default)({
        initialize: true
    }, meta));
}

exports.initializeInputs = _initializeInputs;
var bindActions = exports.bindActions = function bindActions(inputConfig) {
    return {
        setInputs: function setInputs(update, meta) {
            return _setInputs3(inputConfig, update, meta);
        },
        updateAndValidate: function updateAndValidate(update, meta) {
            return _updateAndValidate(inputConfig, update, meta);
        },
        validateInputs: function validateInputs(inputKeys, meta) {
            return _validateInputs(inputConfig, inputKeys, meta);
        },
        setValues: function setValues(newValues, meta) {
            return _setValues(inputConfig, newValues, meta);
        },
        setErrors: function setErrors(inputKeys, meta) {
            return _setErrors(inputConfig, inputKeys, meta);
        },
        resetInputs: function resetInputs(inputKeys, meta) {
            return _resetInputs(inputConfig, inputKeys, meta);
        },
        initializeInputs: function initializeInputs(update, meta) {
            return _initializeInputs(inputConfig, update, meta);
        }
    };
};