'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createOnChangeWithTransform = undefined;

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _objectWithoutProperties2 = require('babel-runtime/helpers/objectWithoutProperties');

var _objectWithoutProperties3 = _interopRequireDefault(_objectWithoutProperties2);

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _identity2 = require('lodash/identity');

var _identity3 = _interopRequireDefault(_identity2);

var _reactDisplayName = require('react-display-name');

var _reactDisplayName2 = _interopRequireDefault(_reactDisplayName);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var createOnChangeWithTransform = exports.createOnChangeWithTransform = function createOnChangeWithTransform(dispatchChange, onChange) {
  var onChangeTransform = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : _identity3.default;
  var parser = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : _identity3.default;
  var onValidationSuccess = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : _identity3.default;
  var onValidationFail = arguments.length > 5 && arguments[5] !== undefined ? arguments[5] : _identity3.default;
  return function (e) {
    var result = dispatchChange(parser(onChangeTransform(e))).then(onValidationSuccess, onValidationFail);
    onChange && onChange(e);
    return result;
  };
};

/**
 * Higher order component that wraps a given input to be compatible with redux-inputs
 *
 * The wrapped component will be given the following props:
 * - _id - An identifier unique to the input consisting of mount point and input id
 * - value - The current displayed value
 * - error - True if current value is invalid
 * - dispatchChange - callback for updating the value
 * - errorText - String to display to the user of current error
 */
var ReduxInputsWrapper = function ReduxInputsWrapper(WrappedComponent) {
  var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {
    onChangeTransform: _identity3.default
  };

  var Wrapper = function Wrapper(_ref) {
    var id = _ref.id,
        _id = _ref._id,
        value = _ref.value,
        parser = _ref.parser,
        formatter = _ref.formatter,
        dispatchChange = _ref.dispatchChange,
        onChange = _ref.onChange,
        onValidationSuccess = _ref.onValidationSuccess,
        onValidationFail = _ref.onValidationFail,
        otherProps = (0, _objectWithoutProperties3.default)(_ref, ['id', '_id', 'value', 'parser', 'formatter', 'dispatchChange', 'onChange', 'onValidationSuccess', 'onValidationFail']);
    var onChangeTransform = options.onChangeTransform;


    return _react2.default.createElement(WrappedComponent, (0, _extends3.default)({ id: id || _id,
      value: formatter ? formatter(value) : value
      // onChange function to take an event facade, dispatch a change event
      , onChange: createOnChangeWithTransform(dispatchChange, onChange, onChangeTransform, parser, onValidationSuccess, onValidationFail)
    }, otherProps));
  };
  Wrapper.displayName = 'ReduxInputsWrapper(' + (0, _reactDisplayName2.default)(WrappedComponent) + ')';
  Wrapper.propTypes = {
    /**
     * Generated id from getInputProps (mountPoint:key)
     */
    _id: _propTypes2.default.string.isRequired,
    /**
     * Optional override id if you don't want to use default _id from inputConfig key
     */
    id: _propTypes2.default.string,
    /**
     * The logical value of the input from the redux store
     */
    value: _propTypes2.default.any,
    /**
     * Function to create a value from a change event into a value to store
     */
    parser: _propTypes2.default.func,
    /**
     * Function to create a DOM-appropriate string from a value
     */
    formatter: _propTypes2.default.func,
    /**
     * Private function from getInputProps, returns a promise
     */
    dispatchChange: _propTypes2.default.func.isRequired,
    /**
     * Callback for React's native onChange
     */
    onChange: _propTypes2.default.func,
    /**
     * Callback after successful validation and update
     */
    onValidationSuccess: _propTypes2.default.func,
    /**
     * Callback after failed validation
     */
    onValidationFail: _propTypes2.default.func
  };
  return Wrapper;
};

exports.default = ReduxInputsWrapper;