'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

exports.getDefaultInputs = getDefaultInputs;
exports.createInputsReducer = createInputsReducer;

var _mapValues2 = require('lodash/mapValues');

var _mapValues3 = _interopRequireDefault(_mapValues2);

var _forEach2 = require('lodash/forEach');

var _forEach3 = _interopRequireDefault(_forEach2);

var _omit2 = require('lodash/omit');

var _omit3 = _interopRequireDefault(_omit2);

var _invariant = require('invariant');

var _invariant2 = _interopRequireDefault(_invariant);

var _mountPoint = require('../util/mountPoint');

var _actionTypes = require('../actions/actionTypes');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function getDefaultInputState(config) {
    return {
        value: config.defaultValue,
        pristine: true
    };
}

function getDefaultInputs(inputConfig) {
    return (0, _mapValues3.default)((0, _omit3.default)(inputConfig, _mountPoint.REDUX_MOUNT_POINT), getDefaultInputState);
}

function _matchesReduxMountPoint(inputConfig, action) {
    return (0, _mountPoint.getReduxMountPoint)(inputConfig) === (action.meta && action.meta.reduxMountPoint);
}

function _syncStateWithInputConfig(inputConfig, state) {
    var otherInputs = {};
    var nsync = true;

    (0, _forEach3.default)(inputConfig, function (config, key) {
        if (key !== _mountPoint.REDUX_MOUNT_POINT && typeof state[key] === 'undefined') {
            otherInputs[key] = getDefaultInputState(config);
            nsync = false;
        }
    });

    return nsync ? state : (0, _extends3.default)({}, otherInputs, state);
}

function createInputsReducer(inputConfig) {
    (0, _invariant2.default)(inputConfig, '[redux-inputs]: inputConfig must be defined');
    return function () {
        var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : getDefaultInputs(inputConfig);
        var action = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

        if (_matchesReduxMountPoint(inputConfig, action)) {
            switch (action.type) {
                case _actionTypes.SET_INPUT:
                    return (0, _extends3.default)({}, state, action.payload);
            }
        }
        return _syncStateWithInputConfig(inputConfig, state);
    };
}